from http import HTTPStatus
from unittest import mock

import pytest
from django.conf import settings
from django.core.exceptions import SuspiciousOperation
from django.http import HttpRequest, HttpResponse
from django.test.client import AsyncRequestFactory, RequestFactory

from django_whatsapp.views import AsyncWebhookView, WebhookView


class TestWebhookView:
    def test_get_valid_request(self, valid_verify_request: HttpRequest) -> None:
        """Test view with valid request."""
        view = WebhookView()

        got: HttpResponse = view.get(valid_verify_request)

        assert isinstance(got, HttpResponse)
        assert got.status_code == HTTPStatus.OK
        assert got.content == b"1158201444"

    def test_get_invalid_request(self, rf: RequestFactory) -> None:
        """Test view with valid request."""
        view = WebhookView()
        request: HttpRequest = rf.get(
            "/",
            query_params={
                "hub.mode": "verify",
                "hub.challenge": "1158201444",
                "hub.verify_token": "meatyhamhock",
            },
        )

        with mock.patch("django_whatsapp.views.verify_request", side_effect=SuspiciousOperation) as mock_verify_request:
            got: HttpResponse = view.get(request)

        assert isinstance(got, HttpResponse)
        assert got.status_code == HTTPStatus.BAD_REQUEST
        mock_verify_request.assert_called_once_with(request, verify_token=settings.WHATSAPP_VERIFY_TOKEN)


@pytest.mark.asyncio
class TestAsyncWebhookView:
    async def test_get_valid_request(self, valid_verify_request: HttpRequest) -> None:
        """Test view with valid request."""
        view = AsyncWebhookView()

        got: HttpResponse = await view.get(valid_verify_request)

        assert isinstance(got, HttpResponse)
        assert got.status_code == HTTPStatus.OK
        assert got.content == b"1158201444"

    async def test_get_invalid_request(self, async_rf: AsyncRequestFactory) -> None:
        """Test view with valid request."""
        view = AsyncWebhookView()
        request: HttpRequest = async_rf.get(
            "/",
            query_params={
                "hub.mode": "verify",
                "hub.challenge": "1158201444",
                "hub.verify_token": "meatyhamhock",
            },
        )

        with mock.patch("django_whatsapp.views.verify_request", side_effect=SuspiciousOperation) as mock_verify_request:
            got: HttpResponse = await view.get(request)

        assert isinstance(got, HttpResponse)
        assert got.status_code == HTTPStatus.BAD_REQUEST
        mock_verify_request.assert_called_once_with(request, verify_token=settings.WHATSAPP_VERIFY_TOKEN)



from collections.abc import Generator
from typing import Any

import pytest
from django.http import HttpRequest
from django.test import override_settings
from django.test.client import RequestFactory


@pytest.fixture(autouse=True)
def _settings() -> Generator[None, Any, None]:
    with override_settings(WHATSAPP_VERIFY_TOKEN="meatyhamhock"):  # noqa: S106
        yield


@pytest.fixture
def valid_verify_request(rf: RequestFactory) -> HttpRequest:
    """A valid verify request."""
    return rf.get(
        "/",
        query_params={
            "hub.mode": "subscribe",
            "hub.challenge": "1158201444",
            "hub.verify_token": "meatyhamhock",
        },
    )



from http import HTTPStatus

import pytest
from django.core.exceptions import SuspiciousOperation
from django.http import HttpRequest
from django.http.response import HttpResponse
from django.test.client import RequestFactory

from django_whatsapp.utils import verify_request


def test_verify_request_invalid_args(valid_verify_request: HttpRequest) -> None:
    """Test :func:`verify_request` parameter validation."""
    with pytest.raises(ValueError, match="verify_token"):
        verify_request(valid_verify_request, "")


def test_verify_request_invalid_mode(rf: RequestFactory) -> None:
    """Test :func:`verify_request` with an invalid verify mode."""
    request: HttpRequest = rf.get(
        "/",
        query_params={
            "hub.mode": "verify",
            "hub.challenge": "1158201444",
            "hub.verify_token": "meatyhamhock",
        },
    )

    with pytest.raises(SuspiciousOperation, match="mode"):
        verify_request(request, "meatyhamhock")


def test_verify_request_invalid_challenge(rf: RequestFactory) -> None:
    """Test :func:`verify_request` with a non-int challenge."""
    request: HttpRequest = rf.get(
        "/",
        query_params={
            "hub.mode": "subscribe",
            "hub.challenge": "CHALLENGE123",
            "hub.verify_token": "meatyhamhock",
        },
    )

    with pytest.raises(SuspiciousOperation, match="challenge"):
        verify_request(request, "meatyhamhock")


def test_verify_request_invalid_token(rf: RequestFactory) -> None:
    """Test :func:`verify_request` with an invalid verify token."""
    request: HttpRequest = rf.get(
        "/",
        query_params={
            "hub.mode": "subscribe",
            "hub.challenge": "1158201444",
            "hub.verify_token": "chickenbreast",
        },
    )

    with pytest.raises(SuspiciousOperation, match="token"):
        verify_request(request, "meatyhamhock")


def test_verify_request_valid(rf: RequestFactory) -> None:
    """Test :func:`verify_request` with a valid request."""
    request: HttpRequest = rf.get(
        "/",
        query_params={
            "hub.mode": "subscribe",
            "hub.challenge": "1158201444",
            "hub.verify_token": "meatyhamhock",
        },
    )

    response: HttpResponse = verify_request(request, "meatyhamhock")

    assert isinstance(response, HttpResponse)
    assert response.status_code == HTTPStatus.OK
    assert response.content == b"1158201444"



"""Utility functions."""

from http import HTTPStatus

from django.core.exceptions import SuspiciousOperation
from django.http import HttpRequest, HttpResponse


def verify_request(request: HttpRequest, verify_token: str) -> HttpResponse:
    """Verify a WhatsApp Verification Request.

    Args:
        request: the actual HTTP request presumably sent by WhatsApp.
        verify_token: the Verify Token configured in the App Dashboard.
    Returns:
        :class:`HttpResponse` to send back do WhatsApp.
    Raises:
        :exc:`SuspiciousOperation` if request does not seem legit.
    """
    mode: str | None = request.GET.get("hub.mode")
    token: str | None = request.GET.get("hub.verify_token")
    challenge: str | None = request.GET.get("hub.challenge")

    if not verify_token:
        msg = "Invalid or missing verify_token"
        raise ValueError(msg)

    if mode != "subscribe":
        msg = "Invalid request mode"
        raise SuspiciousOperation(msg)

    if (challenge is None) or (not challenge.isnumeric()):
        msg = f"{challenge} is not a valid challenge"
        raise SuspiciousOperation(msg)

    if token != verify_token:
        msg = "Invalid verify token"
        raise SuspiciousOperation(msg)

    return HttpResponse(challenge, status=HTTPStatus.OK)



"""WhatsApp Webhook views."""

from django.conf import settings
from django.core.exceptions import SuspiciousOperation
from django.http import HttpRequest, HttpResponse, HttpResponseBadRequest
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from .utils import verify_request


@method_decorator(csrf_exempt, name="dispatch")
class WebhookView(View):
    """View to handle WhatsApp webhook requests."""

    http_method_names: list[str] = ["get", "post", "options"]  # noqa: RUF012

    def get(self, request: HttpRequest) -> HttpResponse:
        """Handle a WhatsApp Verification Request."""
        try:
            return verify_request(request, verify_token=settings.WHATSAPP_VERIFY_TOKEN)
        except SuspiciousOperation:
            return HttpResponseBadRequest()

    def post(self, request: HttpRequest) -> HttpResponse:
        """Handle a WhatsApp Event Notification."""
        raise NotImplementedError


@method_decorator(csrf_exempt, name="dispatch")
class AsyncWebhookView(View):
    """View to handle WhatsApp webhook requests."""

    http_method_names: list[str] = ["get", "post", "options"]  # noqa: RUF012

    async def get(self, request: HttpRequest) -> HttpResponse:
        """Handle a WhatsApp Verification Request."""
        try:
            return verify_request(request, verify_token=settings.WHATSAPP_VERIFY_TOKEN)
        except SuspiciousOperation:
            return HttpResponseBadRequest()

    def post(self, request: HttpRequest) -> HttpResponse:
        """Handle a WhatsApp Event Notification."""
        raise NotImplementedError
